import util

import patterns

import string
pattern_realizations = {} # deprecate
pattern_assertions = {}
all_pattern_realizations = {}
from te_gen_util import *

"""For generating .te and .fc files from pattern instances"""
class PatternAssertion:
    """Equivalence class of pattern constellations all with the same
    security context"""
    def __init__(self,patternstr, specstr, violatorp):
        self.accepted = 1 # innocent until proven guilty
        self.evidence = {} # a set of uids for PatternR's
        self.print_form = specstr
        self.violatorp = violatorp
        self.pattern = patternstr
        self.uid = "pa" + uid_gen.bump()
        uid2name[self.uid] = specstr
        pattern_assertions[specstr] = self

def pattern_assertion_factory (spec, evidence, violatorp):
    there = [pa for pa in pattern_assertions.values() if pa.print_form == spec]
    if len(there) != 0:
        found = there[0]
        found.evidence.append(evidence.uid)
        return found
    else:
        found = PatternAssertion(evidence.pattern, spec, violatorp)
        found.evidence = [evidence.uid]
        return found

class PatternR:
    """ Individual pattern constellation"""
    elements = []
    active = 1
    requires_resource_types = 0
    def te(self):
        """returns line feed delimited string
         When invoked args will be strings"""
        ans = ""
        for c in self.elements:
              ans = ans + (c.te() + '\n')
        return ans
    def spec(self):
        """Returns Python Instantiations.
        Args are resources"""
        patname = get_type(self)[:-1]
        participants = eval('patterns.' + patname).participants
        #print participants
        toreturn = macro_name(patname) + '('
        for p in participants:
            obj = getattr(self, p)
            add_type = ""
            objtype = get_type(obj)
            if self.requires_resource_types and objtype not in ['Process']:
                add_type = ',' + objtype.lower() 
            toreturn = toreturn + ' ' + obj.security_context.split(':')[-1] + add_type + ","
        toreturn = toreturn[:-1] + ') \n'
        return toreturn

def macro_name(s):
    """Given a camelcase string naming some pattern, return the macro
    name which represents appropriate policy for that pattern.

    For example:

    >>> macro_name('SingleProcessMediator')
    'polgen_single_process_mediator'
    """
    accum = ['polgen']
    for c in s:
        if c in string.uppercase:
            accum.append('_')
            accum.append(string.lower(c))
        else:
            accum.append(c)
    return ''.join(accum)

class Declare(PatternR):
    def __init__(self, new_type, ty, evidence):
        self.ty = ty
        self.new_type = new_type
        self.evidence = evidence
        pat_realizations.append(self)
    def te(self):
        return 'type ' + self.new_type + ',' + self.ty + ';'

class CanUnixConnectR (PatternR):
    def __init__(self, app, socket, registerp = 1):
          self.app = app
          self.socket = socket
          if registerp:
                pat_realizations.append(self)

    def spec(self):
        return "CanUnixConnectR('" + self.app.security_context  + "')"

    def te(self):
      return write_te_macro('can_unix_connect', [self.app, self.socket])

class DontAuditR (PatternR):
    def __init__(self, app, fl, fl_type, permissions, registerp = 1):
          self.app = app
          self.fl = fl
          self.fl_type = fl_type
          self.permissions = permissions
          if registerp:
                pat_realizations.append(self)

    def spec(self):
        return "DontAuditR('" + self.app.security_context + "','" + self.fl.security_context + "')"

    def te(self):
      return write_te_macro('dontaudit', [self.app, self.fl])

class CanExecR (PatternR):
     def __init__(self,parent,spawned, registerp = 1):
          self.parent = parent
          self.spawned = spawned
          if registerp:
                pat_realizations.append(self)

     def spec(self):
        return "CanExecR('" + self.parent.security_context + "','" + self.spawned.security_context + "')"

     def te(self):
      return write_te_macro('can_exec', [self.parent, self.spawned])

class AllowR (PatternR):
     def __init__(self,process,holder,holder_type,action, registerp = 1):
          self.process = process
          self.holder = holder
          self.holder_type = holder_type
          if holder_type == 'Socket':
              self.action = 'create_socket_perms'
              self.holder_type = 'unix_stream_socket'

          elif action == 'read' and holder_type == 'File':
               self.action = '{read getattr}'
          elif action == 'read' and holder_type == 'Dir':
              self.action = '{getattr search}'
          elif action == 'open':
              self.action = '{getattr search}'
          else:
              self.action = action
          if registerp:
                pat_realizations.append(self)

     def spec(self):
        if self.process.security_context == None:
            self.process.security_context = 'no_context'
        if self.holder.security_context== None:
            self.holder_security_context = 'no_context'
        return "AllowR('" + self.process.security_context + \
               "','" + self.holder.security_context + "','" + self.holder_type.lower() + \
               "','" + self.action + "')"
     def te(self):
        return "allow " + context2type(self.process) + \
               " " + context2type(self.holder) + ": " \
               + self.holder_type.lower() + " " + self.action + ";"

class PipelineR (PatternR):
     def __init__(self,begin,r1,middle,r2,end, registerp = 1):
          self.requires_resource_types = 1
          self.begin = begin
          self.r1 = r1
          self.middle = middle
          self.r2 = r2
          self.end = end
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(begin, r1, get_type(r1), 'write', 0),
                           AllowR(middle, r1, get_type(r1), '{getattr read}', 0),
                           AllowR(middle, r2, get_type(r2), 'write', 0),
                           AllowR(end, r2, get_type(r2), '{getattr read}',0)]

class PipeR (PatternR):
     def __init__(self,parent,pipe,child, registerp = 1):
          self.parent = parent
          self.pipe = pipe
          self.child = child
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(parent, pipe, 'fifo_file', \
                    '{read write getattr}', 0),
                      CanExecR(parent, child, 0)]

class HubAndSpokeR (PatternR):
     def __init__(self, initiator, r2, colleague, r3, center, registerp = 1):
          self.initiator = initiator
          self.r2 = r2
          self.colleague = colleague
          self.r3 = r3
          self.center = center
          if registerp:
                pat_realizations.append(self)
          self.elements = []#CanExecR(initiator, colleague, 0),
                      #CanExecR(initiator, center, 0)]
          
class ProxyR (PatternR):
     def __init__(self,proxy,r2,service,r3,client, registerp = 1):
          self.requires_resource_types = 1
          self.proxy = proxy
          self.r2 = r2
          self.service = service
          self.r3 = r3
          self.client = client
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(proxy, r2, get_type(r2), 'write', 0),
                           AllowR(service, r2, get_type(r2), '{read getattr}', 0),
                           AllowR(proxy, r3, get_type(r3), '{read getattr}', 0),
                           AllowR(service, r3, get_type(r3), 'write', 0)]

class ObserverR (PatternR):
     def __init__(self,observer,service,client, registerp = 1):
          self.observer = observer
          self.service = service
          self.client = client
          if registerp:
                pat_realizations.append(self)

class LoggerR (PatternR):
     def __init__(self,application,log_file, registerp = 1):
          self.application = application
          self.log_file = log_file
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, log_file, 'file', '{create append}', 0)]

class LibSelinuxInitR (PatternR):
     def __init__(self,application,mounts,new_access, registerp = 1):
          self.application = application
          self.mounts = mounts
          self.new_access = new_access
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, mounts, 'dir', 'search', 0), \
                           AllowR(application, mounts, 'file', '{getattr read}', 0)]
          self.alternate_elements = [DontAudit(application, mounts, 'file', 're_file_perms')]

class SharedLibraryR (PatternR):
     def __init__(self,application,lib_read, registerp = 1):
          self.application = application
          self.lib_read = lib_read
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, lib_read, 'file', '{getattr read}', 0)]

class TempUseR (PatternR):
     def __init__(self,application,temp_file, registerp = 1):
          self.application = application
          self.temp_file = temp_file
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, temp_file, 'file', '{getattr read write}', 0)]

class ConfigR (PatternR):
     def __init__(self,application,config_file, registerp = 1):
          self.application = application
          self.config_file = config_file
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, config_file, 'file', '{getattr read}', 0)]

class CanCheckSelinuxR (PatternR):
     def __init__(self,main,filesystems,current, registerp = 1):
          self.main = main
          self.filesystems = filesystems
          self.current = current
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(main, filesystems, 'file, {getattr read}', 0),
                           AllowR(main, current, 'file', '{getattr read}',0)]

class ActiveObjectR (PatternR):
     def __init__(self,client,waitress,cook, registerp = 1):
          self.client = client
          self.waitress = waitress
          self.cook = cook
          if registerp:
                pat_realizations.append(self)

class WebServerComponentR (PatternR):
     def __init__(self,server,r1,shell,r2,r3,script, registerp = 1):
          self.server = server
          self.r1 = r1
          self.shell = shell
          self.r2 = r2
          self.r3 = r3
          self.script = script
          if registerp:
                pat_realizations.append(self)

class ReactorR (PatternR):
     def __init__(self,reactor,demultiplexer,event_handler,handle, registerp = 1):
          self.reactor = reactor
          self.demultiplexer = demultiplexer
          self.event_handler = event_handler
          self.handle = handle
          if registerp:
                pat_realizations.append(self)

class InterpreterR (PatternR):
     def __init__(self,application,code_file,interpreter, registerp = 1):
          self.application = application
          self.code_file = code_file
          self.interpreter = interpreter
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(application, code_file, 'file', 'rx_file_perms',0),
                           AllowR(application, interpreter, 'file', '{getattr execute}',0)]

class MediatorR (PatternR):
     def __init__(self,mediator,r2,r3,colleague, registerp = 1):
          self.mediator = mediator
          self.r2 = r2
          self.r3 = r3
          self.colleague = colleague
          if registerp:
                pat_realizations.append(self)

class ClientServerR (PatternR):
     def __init__(self,client, socket, server, registerp = 1):
          self.requires_resource_types = 1
          self.server = server
          self.socket = socket
          self.client = client
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(client, socket, get_type(socket), '{create connect read getattr write setattr}', 0),
                           AllowR(server, socket, get_type(socket), '{bind listen accept read write}', 0)]

class ExecutableR (PatternR):
     def __init__(self, application, file, process, registerp = 1):
          self.application = application
          self.file = file
          self.process = process
          if registerp:
                pat_realizations.append(self)
                
class ClientR (PatternR):
     def __init__(self,client, socket,  registerp = 1):
          self.requires_resource_types = 1
          self.socket = socket
          self.client = client
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(client, socket, get_type(socket), '{create connect read getattr write setattr}', 0)]

class SingleProcessMediatorR (PatternR):
     def __init__(self,mediator,r1, registerp = 1):
          self.requires_resource_types = 1
          self.mediator = mediator
          self.r1 = r1
          if registerp:
                pat_realizations.append(self)
          self.elements = [AllowR(mediator, r1, get_type(r1), '{read getattr write}',0)]
